%	ArrestCoefficient XTension
%	
%	Copyright (c) 2016 Chris Barilla
%
%  Installation:
%
%  - Copy this file into the XTensions folder in the Imaris installation directory.
%  - You will find this function in the Image Processing menu
%
%    <CustomTools>
%      <Menu>
%        <Item name="Arrest Coefficient" icon="Matlab" tooltip="Calculate arrest coefficient of Surfaces, make plot, and store data as statistic">
%          <Command>MatlabXT::ArrestCoefficient(%i)</Command>
%        </Item>
%      </Menu>
%    </CustomTools>
% 
%
%	Description: 
%		Get the arrest coefficient of tracked surfaces. The user can set the arrest threshold.
%		A plot of the arrest threshold will be made and the data is saved as a statistic.
%
%


function ArrestCoefficient(aImarisApplicationID)

	% connect to Imaris interface
	if ~isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
	  javaaddpath ImarisLib.jar
	  vImarisLib = ImarisLib;
	  if ischar(aImarisApplicationID)
		aImarisApplicationID = round(str2double(aImarisApplicationID));
	  end
	  imarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
	else
	  imarisApplication = aImarisApplicationID;
	end
	


	% RUN TESTS, these are methods tests in the TestArrestCoefficient.m file
	%TestStops();
	%TestArrestCoeff();
	%TestStatParsingMethod( imarisApplication  );


    % prompt user for which surface collection to evaluate, if more than one collection is in Surpass
	vObject = chooseObjects( imarisApplication );


    % prompt user for speed threshold
	prompt = 'Enter speed threshold value, the maximum speed that is condsidered at rest (default is 0.02 um/s):';
	promptTitle = 'Set Arrest Coefficient Speed Threshold';
	threshold = inputdlg(prompt,promptTitle);
	
	
	% check user input
    if strcmp(threshold, '')
        threshold = 0.02;
    elseif isempty(threshold)
        threshold = 0.02;    	
    else
		threshold = str2double(threshold);
    end

	
	% get the stats from vObject
	stats=vObject.GetStatistics();


	% make struct that maps track Ids to associated track edges
	tracksStruct = getTrackIndicesStruct( vObject );


	% the ids from the stat object are associated with the trackEdge ids,
	% however there is an set shift in the index values that needs to be accounted for.
	mIds = double(sort(unique(stats.mIds)));
	indexShift = mIds(2); % first index should be negative one


    % apply the index shift to the track edges ids, and store in struct
	for i = 1:length(tracksStruct)
		tracksStruct(i).StatIDs =  (tracksStruct(i).ids + indexShift);
	end


	% add speed stat to the struct
	tracksStruct = AddStatToStruct( tracksStruct, 'Speed', stats );


    % add number of stops in the track to the struct
	tracksStruct = AddNumberOfStops(tracksStruct, threshold);

	
	% get the percentage of track edges under threshold
	for i = 1:length(tracksStruct)
		bool = tracksStruct(i).Speed  < threshold;
		num = sum(bool);
		percent = ((num/length(tracksStruct(i).Speed)) * 100);
		tracksStruct(i).ArrestCoefficient = percent;
	end
	

    % the arrest coefficient and number of stops per track as stats to be exported by Imaris
    AddTrackStat( vObject, tracksStruct, 'stops', '', 'Number of Stops in Track' );
    AddTrackStat( vObject, tracksStruct, 'ArrestCoefficient', '%', 'Arrest Coefficient' );



	% make figure
	
    aCoeff = cat(1,tracksStruct.ArrestCoefficient);
    speeds = cat(1,tracksStruct.Speed);
    stops = cat(1,tracksStruct.stops);


	figure
	
	subplot(2,2,1)
	histogram(speeds)
	mu=mean(speeds); %The mean
	hold on
	p1 = plot([mu,mu],ylim,'r--','LineWidth',2);
	legend([p1],' x =  mean')
	title('Distribution of Speeds')
	xlabel(['Velocity (um/s^2)'])
	ylabel('Frequency')
	hold off
	
	subplot(2,2,2)
	cdfplot(speeds)
	mu=mean(speeds); %The mean
	hold on
	p1 = plot([mu,mu],ylim,'r--','LineWidth',2);
	%legend([p1],' mean')
	title('Cumulative Density of Speeds')
	xlabel(['Velocity (um/s^2)'])
	ylabel('Cumulative Frequency')
	hold off

	subplot(2,2,3)
	%boxplot(stops)
	%notBoxPlot(stops)
    histogram(stops, max(stops))
    %xlim([0 max(stops)])
    mu=mean(stops); %The mean
	hold on
	p1 = plot([mu,mu],ylim,'r--','LineWidth',2);
	%legend([p1],' mean')
    title('Number of Stops')
	xlabel('Stops')
	ylabel('Frequency')
	hold off

	subplot(2,2,4)
	histogram(aCoeff,10)
	%histogram(aCoeff,'Normalization','pdf')
	mu=mean(aCoeff); %The mean
	hold on
	p1 = plot([mu,mu],ylim,'r--','LineWidth',2);
	%legend([p1],' mean')
	title('Arrest Coefficient')
	xlabel('Arrest Coefficient')
	ylabel('Frequency')
	hold off
	
	

end


%---------------------------------------------------------%

%
%	chooseObjects function creates a GUI to request the surfaces collection to be returned
%		If there is only one surface collection, that one is returned
%	
%		@param imarisApplication - the IApplication object to be parsed for Surface collection
%

function vObject = chooseObjects( imarisApplication )

	vSurpassScene = imarisApplication.GetSurpassScene;
	nTracks=0;
	surfSelected=0;

	for vChildIndex = 1:vSurpassScene.GetNumberOfChildren
		vDataItem = vSurpassScene.GetChild(vChildIndex - 1);
		if imarisApplication.GetFactory.IsSurfaces(vDataItem)
			nTracks=nTracks+1;
			vSurfaces{nTracks} = imarisApplication.GetFactory.ToSurfaces(vDataItem);    
			SurfNames{nTracks}=char(vSurfaces{nTracks}.GetName);
		end
	end 
	
	if nTracks == 1
		vObject = vSurfaces{1};
	else
		choice = menu('Choose Surface:',SurfNames);
		vObject = vSurfaces{choice};   
	end
	
end


%---------------------------------------------------------%

%
%	getTrackIndicesStruct function creates a structure that contains the track IDs and their associated surface IDs
%	
%		@param vObject - The ISurfaces object to be parsed for track information
%

function tracksStruct = getTrackIndicesStruct( vObject )

	% gets track and surface id info from ISurface Object
    trackids = int32(vObject.GetTrackIds);
	trackedges = vObject.GetTrackEdges;
	tracks = [trackids trackedges];

	
    trackIDs = unique(trackids);
    tracksStruct(length(trackIDs)) = struct();
    
    % iterate through trackIDs getting the associated surface ids
    for i = 1:length(trackIDs)
        inds = tracks(:,1) == trackIDs(i);
        tracksi = tracks(inds,:);
        ids = nan( size(tracksi,1)*2, 1);
        for j = 1:size(tracksi,1)
            ids(j:j+1) = tracksi(j,2:3);
        end
        ids = ids(~isnan(ids));
        ids = sort(unique(ids));
        
        % add Surface IDs and Track ID to structure
        tracksStruct(i).TrackID = trackIDs(i);
        tracksStruct(i).ids = ids;
    end
   
end


%---------------------------------------------------------%

%
%	getStatIndices finds the indices from stats that are associated with
%	surfaces ids and statistic of interest
%
%		@param stats - vObject.GetStatisics()
%		@param stat - statistic of interest in stats
%		@param ids - the track edges surface ids
%

function statInds = GetStatIndices( stats, stat, ids )

	% shift surface ids to maatch surface ids returns from GetStatisics()
	uids = sort(unique(stats.mIds));
    indexShift = uids(2);
    ids = ids + double(indexShift);

    % find the indices in stats that are associated with ids
    j = 1;
    indices = nan(length(stats.mIds), 1);
    for i = 1:length(ids)
        inds = find( stats.mIds == ids(i) );
        if ~isempty(inds)
            indices(j:(j+length(inds)-1)) = inds;
            j = j + length(inds);
        end
    end
    objInds = indices( ~isnan(indices) );
	
	
    % find the indices from stats associated with stat
    mns = cellstr(char(stats.mNames));
    statInds1 = nan(length(mns), 1);
    ix = 1;
    for i = 1:length(mns)
        if strcmp(mns(i), stat)
            statInds1(ix) = i;
            ix = ix + 1;
        end
    end
    statInds1 = statInds1( ~isnan(statInds1) );
    
    % get intersect, so that we have indices associated with both stat and ids
    statInds = intersect( statInds1, objInds );

end


%---------------------------------------------------------%

%
% 	AddStatToStruct function 
%
%		@param tracksStruct
%		@param stat - name of field in tracksStruct
%		@param stats - from vObject.GetStatisics()
%

function tracksStruct = AddStatToStruct( tracksStruct, stat, stats );
	
	ids = sort(unique(cat(1,tracksStruct.ids)));
	statInds = GetStatIndices( stats, stat, ids );
	statIDs = stats.mIds( statInds );
	values = stats.mValues( statInds );
	unit = char(stats.mUnits(statInds(1)));	
	
	for i = 1:length(tracksStruct)
		ar = nan(length(tracksStruct(i).ids),1);
		for j = 1:length(tracksStruct(i).ids)
			ind = find( statIDs == (tracksStruct(i).StatIDs(j)) ); % only take first one, there might be multiple
			ar(j) = values(ind(1));
		end
		tracksStruct = setfield( tracksStruct, {i}, stat, ar);
	end
	
end


%---------------------------------------------------------%

%
% 	AddNumberOfStops function 
%
%		@param tracksStruct
%		@param threshold - the maximum speed to be considered still at rest
%

function tracksStruct = AddNumberOfStops(tracksStruct, threshold) 
	
	for i = 1:length(tracksStruct)
		stopped = tracksStruct(i).Speed(1) <= threshold;
		num = 0;
		for j = 2:length(tracksStruct(i).Speed)
			if tracksStruct(i).Speed(j) > threshold
				if stopped == 1
					num = num + 1;
					stopped = 0;
				end
			else
				stopped = 1;
			end
		end
		
		if stopped == 1
			num = num + 1;
		end
		tracksStruct(i).stops = num;
	end
	
end


%---------------------------------------------------------%

%
% 	AddTrackStat function 
%
%		@param vObject
%		@param tracksStruct
%		@param stat - name of field in the tracksStruct
%		@param units - the unit of stat
%		@param StatName - the name to shown in Imaris stats tab
%

function AddTrackStat( vObject, tracksStruct, stat, units, StatName )

	nTracks = length(tracksStruct);
	trackids = nan( nTracks, 1 );
	values =  nan( nTracks, 1 );
	
	% get the values of the stat of interest
	values = cat(1, getfield(tracksStruct, stat) );
	for i = 1:length(tracksStruct)
		value = getfield(tracksStruct,{i},stat);
		trackid = getfield(tracksStruct,{i},'TrackID');		
		values(i) = value;
		trackids(i) = trackid;
	end
	
	
	% Make a factors matrix with all the features to be added to statistic
	statFactors = cell(4, nTracks);
	statFactors(1, :) = repmat({'Surface'}, [nTracks 1]);
	statFactors(2, :) = repmat({''}, [nTracks 1]);
	statFactors(3, :) = repmat({''}, [nTracks 1]);
	statFactors(4, :) = num2cell(trackids);
	statFactors(4, :) = cellfun(@num2str, statFactors(2, :), 'UniformOutput', 0);
    factorNames = {'Category', 'Collection', 'Time', 'Track_ID'};
    
	
	% make array with the unit for the angles
	statUnits = repmat({units}, [nTracks 1]);

	% Make names of the statistics
	statNames = repmat({StatName}, [nTracks 1]); 

	% Add the stats
    vObject.AddStatistics(statNames, values, statUnits, ...
                        statFactors, factorNames, trackids)
    
end






	




